# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from setuptools import setup, find_packages
import os
import sys

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='lale',
    version='0.3.0',
    author="Avraham Shinnar, Guillaume Baudart, Kiran Kate, Martin Hirzel",
    description="Language for Automated Learning Exploration",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/IBM/lale",
    python_requires='>=3.6',
    packages=find_packages(),
    license='',
    install_requires=[
        'astunparse',
        'graphviz',
        'hyperopt==0.1.2',
        'jsonschema==2.6.0',
        'jsonsubschema',
        'numpy',
        'PyYAML',
        'scikit-learn==0.20.3',
        'scipy',
        'pandas',
        'xgboost',
        'lightgbm',
        'decorator',
        'torch>=1.0',
        'h5py'],
      extras_require={
          'full': [
              'pytorch-pretrained-bert>=0.6.1',
              'torchvision>=0.2.2',
              'liac-arff>=2.4.0',
              'tensorflow-datasets>=1.0.1',
              'tensorflow>=1.13.1',
              'tensorflow_hub',
              'spacy',
              'smac<=0.10.0',
              'aif360',
              'numba',
              'BlackBoxAuditing'],
          'test':[
              'jupyter',
              'mypy',
              'flake8',
              'numpydoc',
              'sphinx',
              'm2r',
              'sphinx_rtd_theme',
              'sphinxcontrib.apidoc'
          ]}
)



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


import sklearn.impute
import lale.helpers
import lale.operators
import numpy as np


class SimpleImputerImpl():

    def __init__(self, missing_values=None, strategy='mean', fill_value=None, verbose=0, copy=True):
        self._hyperparams = {
            'missing_values': missing_values,
            'strategy': strategy,
            'fill_value': fill_value,
            'verbose': verbose,
            'copy': copy}

    def fit(self, X, y=None):
        self._sklearn_model = sklearn.impute.SimpleImputer(**self._hyperparams)
        self._sklearn_model.fit(X, y)
        return self

    def transform(self, X):
        return self._sklearn_model.transform(X)


_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Imputation transformer for completing missing values.',
    'allOf': [{
        'type': 'object',
        'additionalProperties': False,
        'required': ['missing_values', 'strategy', 'fill_value', 'verbose', 'copy'],
        'relevantToOptimizer': ['strategy'],        
        'properties': {
            'missing_values': {
                'anyOf': [{'type': 'number'},
                          {'type': 'string'},
                          {'enum': [np.nan]},
                          {'enum': [None]}],
                'default': np.nan,
                'description': 'The placeholder for the missing values.'},
            'strategy': {
                'anyOf': [
                    {'enum':['constant'], 'forOptimizer': False}, 
                    {'enum': ['mean', 'median', 'most_frequent']}],
                'default': 'mean',
                'description': 'The imputation strategy.'},
            'fill_value': {
                'anyOf': [{'type': 'number'},
                          {'type': 'string'},
                          {'enum': [None]}],
                'default': None,
                'description': 'When strategy == "constant", fill_value is used to replace all occurrences of missing_values'},
            'verbose': {
                'type': 'integer',
                'default': 0,
                'description': 'Controls the verbosity of the imputer.'},
            'copy': {
                'type': 'boolean',
                'default': True,
                'description': 'If True, a copy of X will be created. If False, imputation will'},
        }},
        {'description': "fill_value, only used when strategy='constant'",
         'anyOf': [{
             'type': 'object',
             'properties': {
                 'strategy': {
                     'enum': ['constant']},
             }}, {
             'type': 'object',
             'properties': {
                 'fill_value': {
                     'enum': [None]},
             }}]}]
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit the imputer on X.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array', 
                'items':{
                    'anyOf':[
                    {'type': 'number'},
                    {'type': 'string'}]}},
            'description': 'Input data, where ``n_samples`` is the number of samples and  ``n_features`` is the number of features.'},
    },
}
_input_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Impute all missing values in X.',
    'type': 'object',
    'required': ['X'],
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array', 
                'items':{
                    'anyOf':[
                    {'type': 'number'},
                    {'type': 'string'}]}},
            'description': 'The input data to complete.'},
    },
}
_output_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The input data to complete.',
    'type': 'array',
    'items': {'type': 'array', 
        'items':{
            'anyOf':[
                {'type': 'number'},
                {'type': 'string'}]}},
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Combined schema for expected data and hyperparameters.',
    'documentation_url': 'https://scikit-learn.org/stable/modules/generated/sklearn.impute.SimpleImputer.html',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_transform_schema,
        'output': _output_transform_schema},
}
if (__name__ == '__main__'):
    lale.helpers.validate_is_schema(_combined_schemas)
SimpleImputer = lale.operators.make_operator(SimpleImputerImpl, _combined_schemas)



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest
import jsonschema
import warnings
from lale.lib.lale import ConcatFeatures
from lale.lib.lale import NoOp
from lale.lib.sklearn import KNeighborsClassifier
from lale.lib.sklearn import LinearSVC
from lale.lib.sklearn import LogisticRegression
from lale.lib.sklearn import MinMaxScaler
from lale.lib.sklearn import Normalizer
from lale.lib.sklearn import MLPClassifier
from lale.lib.sklearn import Nystroem
from lale.lib.sklearn import OneHotEncoder
from lale.lib.sklearn import PCA
from lale.lib.sklearn import TfidfVectorizer
from lale.lib.sklearn import MultinomialNB
from lale.lib.sklearn import SimpleImputer
from lale.lib.sklearn import SVC
from lale.lib.xgboost import XGBClassifier
from lale.lib.sklearn import PassiveAggressiveClassifier
from lale.lib.sklearn import StandardScaler
from lale.lib.sklearn import FeatureAgglomeration

from lale.search.SMAC import get_smac_space, lale_trainable_op_from_config

import numpy as np
from typing import List

def test_f_min(op, X, y, num_folds=5):
    from sklearn import datasets
    from lale.helpers import cross_val_score
    import numpy as np

    # try:
    scores = cross_val_score(op, X, y, cv = num_folds)

    return 1-np.mean(scores)  # Minimize!
    # except BaseException as e:
    #     print(e)
    #     return 

def test_iris_f_min(op, num_folds=5):
    from sklearn import datasets

    iris = datasets.load_iris()
    return test_f_min(op, iris.data, iris.target, num_folds = num_folds)

def test_iris_f_min_for_folds(num_folds=5):
    return lambda op: test_iris_f_min(op, num_folds=num_folds)
    
from lale.search.SMAC import lale_op_smac_tae

def test_iris_fmin_tae(op, num_folds=5):
    return lale_op_smac_tae(op, test_iris_f_min_for_folds(num_folds=num_folds))

class TestSMAC(unittest.TestCase):
    def test_smac(self):

        import numpy as np
        from sklearn import svm, datasets
        from sklearn.model_selection import cross_val_score

        # Import ConfigSpace and different types of parameters
        from smac.configspace import ConfigurationSpace

        # Import SMAC-utilities
        from smac.tae.execute_func import ExecuteTAFuncDict
        from smac.scenario.scenario import Scenario
        from smac.facade.smac_facade import SMAC

        from lale.search.SMAC import get_smac_space

        lr = LogisticRegression()

        cs:ConfigurationSpace = get_smac_space(lr)

        # Scenario object
        scenario = Scenario({"run_obj": "quality",   # we optimize quality (alternatively runtime)
                            "runcount-limit": 1,  # maximum function evaluations
                            "cs": cs,               # configuration space
                            "deterministic": "true",
                            "abort_on_first_run_crash": False
                            })

        # Optimize, using a SMAC-object
        tae = test_iris_fmin_tae(lr, num_folds=2)
        print("Optimizing! Depending on your machine, this might take a few minutes.")
        smac = SMAC(scenario=scenario, rng=np.random.RandomState(42),
                tae_runner=tae)

        incumbent = smac.optimize()

        inc_value = tae(incumbent)

        print("Optimized Value: %.2f" % (inc_value))

    def dont_test_smac_choice(self):

        import numpy as np
        from sklearn import svm, datasets
        from sklearn.model_selection import cross_val_score

        # Import ConfigSpace and different types of parameters
        from smac.configspace import ConfigurationSpace

        # Import SMAC-utilities
        from smac.tae.execute_func import ExecuteTAFuncDict
        from smac.scenario.scenario import Scenario
        from smac.facade.smac_facade import SMAC


        tfm = PCA() | Nystroem() | NoOp()
        planned_pipeline1 = (OneHotEncoder(handle_unknown = 'ignore',  sparse = False) | NoOp()) >> tfm >> (LogisticRegression() | KNeighborsClassifier())

        cs:ConfigurationSpace = get_smac_space(planned_pipeline1, lale_num_grids=5)

        # Scenario object
        scenario = Scenario({"run_obj": "quality",   # we optimize quality (alternatively runtime)
                            "runcount-limit": 1,  # maximum function evaluations
                            "cs": cs,               # configuration space
                            "deterministic": "true"
                            })

        # Optimize, using a SMAC-object
        tae = test_iris_fmin_tae(planned_pipeline1, num_folds=2)
        print("Optimizing! Depending on your machine, this might take a few minutes.")
        smac = SMAC(scenario=scenario, rng=np.random.RandomState(42),
                tae_runner=tae)

        incumbent = smac.optimize()

        inc_value = tae(incumbent)

        print("Optimized Value: %.2f" % (inc_value))
        
def run_hyperopt_on_planned_pipeline(planned_pipeline, max_iters=1) :
    # data
    from sklearn.datasets import load_iris
    features, labels = load_iris(return_X_y=True)
    # set up optimizer
    from lale.lib.lale.hyperopt_classifier import HyperoptClassifier
    opt = HyperoptClassifier(model=planned_pipeline, max_evals=max_iters)
    # run optimizer
    res = opt.fit(features, labels)    

class TestHyperoptOperatorDuplication(unittest.TestCase) :
    def test_planned_pipeline_1(self) :
        plan = (
            ( PCA & ( MinMaxScaler | Normalizer ) ) >> ConcatFeatures() >>
            ( MinMaxScaler | Normalizer ) >>
            ( LogisticRegression | KNeighborsClassifier)
        )
        run_hyperopt_on_planned_pipeline(plan)

    def test_planned_pipeline_2(self) :
        plan = (
            ( MinMaxScaler() & NoOp() ) >> ConcatFeatures() >>
            ( NoOp() & MinMaxScaler() ) >> ConcatFeatures() >>
            ( LogisticRegression | KNeighborsClassifier )
        )
        run_hyperopt_on_planned_pipeline(plan)

    def test_planned_pipeline_3(self) :
        plan = (
            ( MinMaxScaler() & NoOp() ) >> ConcatFeatures() >>
            ( StandardScaler & ( NoOp() | MinMaxScaler() ) ) >> ConcatFeatures() >>
            ( LogisticRegression | KNeighborsClassifier )
        )
        run_hyperopt_on_planned_pipeline(plan)




from sklearn.preprocessing.data import MinMaxScaler as SKLModel
import lale.helpers
import lale.operators
from numpy import nan, inf

class MinMaxScalerImpl():

    def __init__(self, feature_range=(0, 1), copy=True):
        self._hyperparams = {
            'feature_range': feature_range,
            'copy': copy}

    def fit(self, X, y=None):
        self._sklearn_model = SKLModel(**self._hyperparams)
        if (y is not None):
            self._sklearn_model.fit(X, y)
        else:
            self._sklearn_model.fit(X)
        return self

    def transform(self, X):
        return self._sklearn_model.transform(X)
_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'inherited docstring for MinMaxScaler    Transforms features by scaling each feature to a given range.',
    'allOf': [{
        'type': 'object',
        'required': ['feature_range', 'copy'],
        'relevantToOptimizer': ['copy'],
        'additionalProperties': False,
        'properties': {
            'feature_range': {
                'XXX TODO XXX': 'tuple (min, max), default=(0, 1)',
                'description': 'Desired range of transformed data.',
                'type': 'array',
                'typeForOptimizer': 'tuple',
                'default': (0, 1)},
            'copy': {
                'type': 'boolean',
                'default': True,
                'description': 'Set to False to perform inplace row normalization and avoid a'},
        }}],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Compute the minimum and maximum to be used for later scaling.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The data used to compute the per-feature minimum and maximum'},
    },
}
_input_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Scaling features of X according to feature_range.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'Input data that will be transformed.'},
    },
}
_output_transform_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Scaling features of X according to feature_range.',
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Combined schema for expected data and hyperparameters.',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['transformer'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_transform': _input_transform_schema,
        'output_transform': _output_transform_schema},
}
if (__name__ == '__main__'):
    lale.helpers.validate_is_schema(_combined_schemas)
MinMaxScaler = lale.operators.make_operator(MinMaxScalerImpl, _combined_schemas)





from sklearn.neural_network.multilayer_perceptron import MLPRegressor as SKLModel
import lale.helpers
import lale.operators
from numpy import nan, inf

class MLPRegressorImpl():

    def __init__(self, hidden_layer_sizes=(100,), activation='relu', solver='adam', alpha=0.0001, batch_size='auto', learning_rate='constant', learning_rate_init=0.001, power_t=0.5, max_iter=200, shuffle=True, random_state=None, tol=0.0001, verbose=False, warm_start=False, momentum=0.9, nesterovs_momentum=True, early_stopping=False, validation_fraction=0.1, beta_1=0.9, beta_2=0.999, epsilon=1e-08, n_iter_no_change=10):
        self._hyperparams = {
            'hidden_layer_sizes': hidden_layer_sizes,
            'activation': activation,
            'solver': solver,
            'alpha': alpha,
            'batch_size': batch_size,
            'learning_rate': learning_rate,
            'learning_rate_init': learning_rate_init,
            'power_t': power_t,
            'max_iter': max_iter,
            'shuffle': shuffle,
            'random_state': random_state,
            'tol': tol,
            'verbose': verbose,
            'warm_start': warm_start,
            'momentum': momentum,
            'nesterovs_momentum': nesterovs_momentum,
            'early_stopping': early_stopping,
            'validation_fraction': validation_fraction,
            'beta_1': beta_1,
            'beta_2': beta_2,
            'epsilon': epsilon,
            'n_iter_no_change': n_iter_no_change}

    def fit(self, X, y=None):
        self._sklearn_model = SKLModel(**self._hyperparams)
        if (y is not None):
            self._sklearn_model.fit(X, y)
        else:
            self._sklearn_model.fit(X)
        return self

    def predict(self, X):
        return self._sklearn_model.predict(X)
_hyperparams_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'inherited docstring for MLPRegressor    Multi-layer Perceptron regressor.',
    'allOf': [{
        'type': 'object',
        'required': ['hidden_layer_sizes', 'activation', 'solver', 'alpha', 'batch_size', 'learning_rate', 'learning_rate_init', 'power_t', 'max_iter', 'shuffle', 'random_state', 'tol', 'verbose', 'warm_start', 'momentum', 'nesterovs_momentum', 'early_stopping', 'validation_fraction', 'beta_1', 'beta_2', 'epsilon', 'n_iter_no_change'],
        'relevantToOptimizer': ['activation', 'solver', 'alpha', 'batch_size', 'learning_rate', 'max_iter', 'shuffle', 'tol', 'nesterovs_momentum', 'epsilon'],
        'additionalProperties': False,
        'properties': {
            'hidden_layer_sizes': {
                'XXX TODO XXX': 'tuple, length = n_layers - 2, default (100,)',
                'description': 'The ith element represents the number of neurons in the ith',
                'type': 'array',
                'typeForOptimizer': 'tuple',
                'default': (100,)},
            'activation': {
                'enum': ['identity', 'logistic', 'tanh', 'relu'],
                'default': 'relu',
                'description': 'Activation function for the hidden layer.'},
            'solver': {
                'enum': ['lbfgs', 'sgd', 'adam'],
                'default': 'adam',
                'description': 'The solver for weight optimization.'},
            'alpha': {
                'type': 'number',
                'minimumForOptimizer': 1e-10,
                'maximumForOptimizer': 1.0,
                'distribution': 'loguniform',
                'default': 0.0001,
                'description': 'L2 penalty (regularization term) parameter.'},
            'batch_size': {
                'anyOf': [{
                    'type': 'integer',
                    'minimumForOptimizer': 3,
                    'maximumForOptimizer': 128,
                    'distribution': 'uniform'}, {
                    'enum': ['auto']}],
                'default': 'auto',
                'description': 'Size of minibatches for stochastic optimizers.'},
            'learning_rate': {
                'enum': ['constant', 'invscaling', 'adaptive'],
                'default': 'constant',
                'description': 'Learning rate schedule for weight updates.'},
            'learning_rate_init': {
                'type': 'number',
                'default': 0.001,
                'description': 'The initial learning rate used. It controls the step-size'},
            'power_t': {
                'type': 'number',
                'default': 0.5,
                'description': 'The exponent for inverse scaling learning rate.'},
            'max_iter': {
                'type': 'integer',
                'minimumForOptimizer': 10,
                'maximumForOptimizer': 1000,
                'distribution': 'uniform',
                'default': 200,
                'description': 'Maximum number of iterations. The solver iterates until convergence'},
            'shuffle': {
                'type': 'boolean',
                'default': True,
                'description': 'Whether to shuffle samples in each iteration. Only used when'},
            'random_state': {
                'anyOf': [{
                    'type': 'integer'}, {
                    'type': 'object'}, {
                    'enum': [None]}],
                'default': None,
                'description': 'If int, random_state is the seed used by the random number generator;'},
            'tol': {
                'type': 'number',
                'minimumForOptimizer': 1e-08,
                'maximumForOptimizer': 0.01,
                'distribution': 'loguniform',
                'default': 0.0001,
                'description': 'Tolerance for the optimization. When the loss or score is not improving'},
            'verbose': {
                'type': 'boolean',
                'default': False,
                'description': 'Whether to print progress messages to stdout.'},
            'warm_start': {
                'type': 'boolean',
                'default': False,
                'description': 'When set to True, reuse the solution of the previous'},
            'momentum': {
                'type': 'number',
                'default': 0.9,
                'description': 'Momentum for gradient descent update.  Should be between 0 and 1. Only'},
            'nesterovs_momentum': {
                'type': 'boolean',
                'default': True,
                'description': "Whether to use Nesterov's momentum. Only used when solver='sgd' and"},
            'early_stopping': {
                'type': 'boolean',
                'default': False,
                'description': 'Whether to use early stopping to terminate training when validation'},
            'validation_fraction': {
                'type': 'number',
                'default': 0.1,
                'description': 'The proportion of training data to set aside as validation set for'},
            'beta_1': {
                'type': 'number',
                'default': 0.9,
                'description': 'Exponential decay rate for estimates of first moment vector in adam,'},
            'beta_2': {
                'type': 'number',
                'default': 0.999,
                'description': 'Exponential decay rate for estimates of second moment vector in adam,'},
            'epsilon': {
                'type': 'number',
                'minimumForOptimizer': 1e-08,
                'maximumForOptimizer': 1.35,
                'distribution': 'loguniform',
                'default': 1e-08,
                'description': "Value for numerical stability in adam. Only used when solver='adam'"},
            'n_iter_no_change': {
                'type': 'integer',
                'default': 10,
                'description': 'Maximum number of epochs to not meet ``tol`` improvement.'},
        }}, {
        'description': "learning_rate, only used when solver='sgd'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'learning_rate': {
                    'enum': ['constant']},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd']},
            }}]}, {
        'description': "learning_rate_init, only used when solver='sgd' or 'adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'learning_rate_init': {
                    'enum': [0.001]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd', 'adam']},
            }}]}, {
        'description': "power_t, only used when solver='sgd'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'power_t': {
                    'enum': [0.5]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd']},
            }}]}, {
        'description': "shuffle, only used when solver='sgd' or 'adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'shuffle': {
                    'enum': [True]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd', 'adam']},
            }}]}, {
        'description': "momentum, only used when solver='sgd'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'momentum': {
                    'enum': [0.9]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd']},
            }}]}, {
        'XXX TODO XXX': "Parameter: nesterovs_momentum > only used when solver='sgd' and momentum > 0"}, {
        'description': "early_stopping, only effective when solver='sgd' or 'adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'early_stopping': {
                    'enum': [False]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd', 'adam']},
            }}]}, {
        'description': 'validation_fraction, only used if early_stopping is true',
        'anyOf': [{
            'type': 'object',
            'properties': {
                'validation_fraction': {
                    'enum': [0.1]},
            }}, {
            'type': 'object',
            'properties': {
                'early_stopping': {
                    'enum': [True]},
            }}]}, {
        'description': "beta_1, only used when solver='adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'beta_1': {
                    'enum': [0.9]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['adam']},
            }}]}, {
        'description': "beta_2, only used when solver='adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'beta_2': {
                    'enum': [0.999]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['adam']},
            }}]}, {
        'description': "epsilon, only used when solver='adam'",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'epsilon': {
                    'enum': [1e-08]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['adam']},
            }}]}, {
        'description': "n_iter_no_change, only effective when solver='sgd' or 'adam' ",
        'anyOf': [{
            'type': 'object',
            'properties': {
                'n_iter_no_change': {
                    'enum': [10]},
            }}, {
            'type': 'object',
            'properties': {
                'solver': {
                    'enum': ['sgd', 'adam']},
            }}]}],
}
_input_fit_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Fit the model to data matrix X and target(s) y.',
    'type': 'object',
    'properties': {
        'X': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'XXX TODO XXX': 'item type'},
                'XXX TODO XXX': 'array-like or sparse matrix, shape (n_samples, n_features)'}, {
                'type': 'array',
                'items': {
                    'type': 'array',
                    'items': {
                        'type': 'number'},
                }}],
            'description': 'The input data.'},
        'y': {
            'anyOf': [{
                'type': 'array',
                'items': {
                    'type': 'number'},
            }, {
                'type': 'array',
                'items': {
                    'type': 'array',
                    'items': {
                        'type': 'number'},
                }}],
            'description': 'The target values (class labels in classification, real numbers in'},
    },
}
_input_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Predict using the multi-layer perceptron model.',
    'type': 'object',
    'properties': {
        'X': {
            'type': 'array',
            'items': {
                'type': 'array',
                'items': {
                    'type': 'number'},
            },
            'description': 'The input data.'},
    },
}
_output_predict_schema = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'The predicted values.',
    'type': 'array',
    'items': {
        'type': 'array',
        'items': {
            'type': 'number'},
    },
}
_combined_schemas = {
    '$schema': 'http://json-schema.org/draft-04/schema#',
    'description': 'Combined schema for expected data and hyperparameters.',
    'type': 'object',
    'tags': {
        'pre': [],
        'op': ['estimator'],
        'post': []},
    'properties': {
        'hyperparams': _hyperparams_schema,
        'input_fit': _input_fit_schema,
        'input_predict': _input_predict_schema,
        'output_predict': _output_predict_schema},
}
if (__name__ == '__main__'):
    lale.helpers.validate_is_schema(_combined_schemas)
MLPRegressor = lale.operators.make_operator(MLPRegressorImpl, _combined_schemas)




# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import pandas as pd
from sklearn.model_selection import train_test_split

def load_iris_df(test_size = 0.2):
    from sklearn.datasets import load_iris
    from sklearn.utils import shuffle
    iris = load_iris()
    X = iris.data
    y = iris.target
    target_name = 'target'
    X, y = shuffle(iris.data, iris.target, random_state=42)
    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=test_size, random_state=42)

    X_train_df = pd.DataFrame(X_train, columns = iris.feature_names)
    y_train_df = pd.Series(y_train, name = target_name)

    X_test_df = pd.DataFrame(X_test, columns = iris.feature_names)
    y_test_df = pd.Series(y_test, name = target_name)

    return (X_train_df, y_train_df), (X_test_df, y_test_df)

def load_digits_df(test_size = 0.2):
    from sklearn.datasets import load_digits
    from sklearn.utils import shuffle
    digits = load_digits()
    X, y = shuffle(digits.data, digits.target, random_state=42)

    target_name = 'target'
    column_names = ['x%d' % i for i in range(X.shape[1])]
    X_train, X_test, y_train, y_test = train_test_split(
        X, y, test_size=test_size, random_state=42)
    X_train_df = pd.DataFrame(X_train, columns = column_names)
    y_train_df = pd.Series(y_train, name = target_name)

    X_test_df = pd.DataFrame(X_test, columns = column_names)
    y_test_df = pd.Series(y_test, name = target_name)

    return (X_train_df, y_train_df), (X_test_df, y_test_df)




from torch.utils.data import Dataset
import h5py

class HDF5TorchDataset(Dataset):
    """Pytorch Dataset subclass that takes a hdf5 file pointer."""

    def __init__(self, file_path):
        """.
        
        Parameters
        ----------
        file : file is an object of class h5py.File
        """
        self.file_path = file_path
        h5_file = h5py.File(file_path)
        self.length = h5_file['X'].shape[0]

    def __len__(self):
        return self.length

    def __getitem__(self, idx):
        with h5py.File(self.file_path) as h5_file:
            X = h5_file['X']
            try:
                y = h5_file['y']
            except KeyError:
                y = None
            if y is None:
                element = X[idx]
            else:
                element = X[idx], y[idx]
        return element

    def get_data(self):
        with h5py.File(self.file_path) as h5_file:
            X = h5_file['X'][:]
            try:
                y = h5_file['y'][:]
            except KeyError:
                y = None
            if y is None:
                return X
            else:
                return X, y



# Copyright 2019 IBM Corporation
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from typing import cast, TypeVar, Any, Dict, List, Tuple, Optional, Union


class Undefined():
    pass


undefined = Undefined()

T = TypeVar('T')
Option = Union[Undefined, T]


class Schema:

    schema: Dict[str, Any]

    def __init__(self,
                 desc: Option[str] = undefined,
                 default: Option[Any] = undefined,
                 forOptimizer: bool = True):
        self.schema: Dict[str, Any] = {}
        if not isinstance(default, Undefined):
            self.schema['default'] = default
        if not isinstance(desc, Undefined):
            self.schema['description'] = desc
        if not forOptimizer:
            self.schema['forOptimizer'] = forOptimizer

    def set(self, prop: str, value: Option[Any]):
        if not isinstance(value, Undefined):
            self.schema[prop] = value


#Base Type

class Bool(Schema):
    def __init__(self,
                 desc: Option[str] = undefined,
                 default: Option[str] = undefined,
                 forOptimizer: bool = True):
        super().__init__(desc, default, forOptimizer)
        self.set('type', 'boolean')


class Enum(Schema):
    def __init__(self,
                 values: List[str] = [],
                 desc: Option[str] = undefined,
                 default: Option[str] = undefined,
                 forOptimizer: bool = True):
        super().__init__(desc, default, forOptimizer)
        self.set('enum', values)


class Float(Schema):
    def __init__(self,
                 desc: Option[str] = undefined,
                 default: Option[str] = undefined,
                 forOptimizer: bool = True,
                 min: Option[float] = undefined,
                 exclusiveMin: Option[bool] = undefined,
                 minForOptimizer: Option[bool] = undefined,
                 max: Option[float] = undefined,
                 exclusiveMax: Option[bool] = undefined,
                 maxForOptimizer: Option[bool] = undefined,
                 distribution: Option[str] = undefined):
        super().__init__(desc, default, forOptimizer)
        self.set('type', 'number')
        self.set('minimum', min)
        self.set('exclusiveMinimum', exclusiveMin)
        self.set('minimumForOptimizer', minForOptimizer)
        self.set('maximum', max)
        self.set('exclusiveMaximum', exclusiveMax)
        self.set('maximumForOptimizer', maxForOptimizer)
        self.set('distribution', distribution)


class Int(Schema):
    def __init__(self,
                 desc: Option[str] = undefined,
                 default: Option[str] = undefined,
                 forOptimizer: bool = True,
                 min: Option[int] = undefined,
                 exclusiveMin: Option[bool] = undefined,
                 max: Option[int] = undefined,
                 exclusiveMax: Option[bool] = undefined,
                 distribution: Option[str] = undefined):
        super().__init__(desc, default, forOptimizer)
        self.set('type', 'integer')
        self.set('minimum', min)
        self.set('exclusiveMinimum', exclusiveMin)
        self.set('maximum', max)
        self.set('exclusiveMaximum', exclusiveMax)
        self.set('distribution', distribution)


class Null(Schema):
    def __init__(self,
                 desc: Option[str] = undefined,
                 forOptimizer: bool = True):
        super().__init__(desc=desc, forOptimizer=forOptimizer)
        self.set('enum', [None])


class Not(Schema):
    def __init__(self,
                 body: Schema):
        super().__init__()
        self.schema = {'not': body.schema}


class JSON(Schema):
    def __init__(self,
                 body: Dict[str, Any]):
        super().__init__()
        self.schema = body

# Combinator


class AnyOf(Schema):
    def __init__(self,
                 types: List[Schema] = [],
                 desc: Option[str] = undefined,
                 default: Option[Any] = undefined):
        super().__init__(desc, default)
        self.set('anyOf', [t.schema for t in types])


class AllOf(Schema):
    def __init__(self,
                 types: List[Schema] = [],
                 desc: Option[str] = undefined,
                 default: Option[Any] = undefined):
        super().__init__(desc, default)
        self.set('allOf', [t.schema for t in types])


class Array(Schema):
    def __init__(self,
                 items: Schema,
                 desc: Option[str] = undefined,
                 default: Option[List[Any]] = undefined,
                 forOptimizer: bool = True,
                 minItems: Option[int] = undefined,
                 minItemsForOptimizer: Option[int] = undefined,
                 maxItems: Option[int] = undefined,
                 maxItemsForOptimizer: Option[int] = undefined,
                 typeForOptimizer: Option[str] = undefined,):
        super().__init__(desc, default, forOptimizer)
        self.set('type', 'array')
        self.set('items', items.schema)
        self.set('minItems', minItems)
        self.set('minItemsForOptimizer', minItemsForOptimizer)
        self.set('maxItems', maxItems)
        self.set('maxItemsForOptimizer', maxItemsForOptimizer)
        self.set('typeForOptimizer', typeForOptimizer)


class Object(Schema):
    def __init__(self,
                 default: Option[Any] = undefined,
                 desc: Option[str] = undefined,
                 forOptimizer: bool = True,
                 required: Option[List[str]] = undefined,
                 additionalProperties: Option[bool] = undefined,
                 **kwargs: Schema):
        super().__init__(desc, default, forOptimizer)
        self.set('type', 'object')
        self.set('required', required)
        self.set('additionalProperties', additionalProperties)
        self.set('properties', {k: p.schema for (k, p) in kwargs.items()})
